/* tcp-web-helper.cc
// TE816 - Modelagem e Avaliação de Desempenho
// Pós Graduação em Engenharia Elétrica
// Universidade Federal do Paraná
// This souce was developed as basis for homework 
//
// -- 10/11/2015
// ==================================
// Prof. Carlos Marcelo Pedroso
// pedroso@eletrica.ufpr.br
// http://www.eletrica.ufpr.br/pedroso
// Universidade Federal do Paraná
// Departamento de Engenharia Elétrica
// Centro Politécnico
// CEP: 81531-990 - Curitiba - PR - Brasil
 */

#include "tcp-web-helper.h"
#include "ns3/tcp-web-server.h"
#include "ns3/tcp-web-client.h"
#include "ns3/uinteger.h"
#include "ns3/names.h"

namespace ns3 {

  
TcpWebServerHelper::TcpWebServerHelper (uint16_t port)
{
  m_factory.SetTypeId (TcpWebServer::GetTypeId ());
  SetAttribute ("Port", UintegerValue (port));
}

void
TcpWebServerHelper::SetAttribute (
  std::string name,
  const AttributeValue &value)
{
  m_factory.Set (name, value);
}

ApplicationContainer
TcpWebServerHelper::Install (Ptr<Node> node) const
{
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
TcpWebServerHelper::Install (std::string nodeName) const
{
  Ptr<Node> node = Names::Find<Node> (nodeName);
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
TcpWebServerHelper::Install (NodeContainer c) const
{
  ApplicationContainer apps;
  for (NodeContainer::Iterator i = c.Begin (); i != c.End (); ++i)
    {
      apps.Add (InstallPriv (*i));
    }

  return apps;
}

Ptr<Application>
TcpWebServerHelper::InstallPriv (Ptr<Node> node) const
{
  Ptr<Application> app = m_factory.Create<TcpWebServer> ();
  node->AddApplication (app);

  return app;
}

TcpWebClientHelper::TcpWebClientHelper (Address address, uint16_t port)
{
  m_factory.SetTypeId (TcpWebClient::GetTypeId ());
  SetAttribute ("RemoteAddress", AddressValue (address));
  SetAttribute ("RemotePort", UintegerValue (port));
}

void
TcpWebClientHelper::SetAttribute (
  std::string name,
  const AttributeValue &value)
{
  m_factory.Set (name, value);
}

void
TcpWebClientHelper::SetFill (Ptr<Application> app, std::string fill)
{
  app->GetObject<TcpWebClient>()->SetFill (fill);
}

void
TcpWebClientHelper::SetFill (Ptr<Application> app, uint8_t fill, uint32_t dataLength)
{
  app->GetObject<TcpWebClient>()->SetFill (fill, dataLength);
}

void
TcpWebClientHelper::SetFill (Ptr<Application> app, uint8_t *fill, uint32_t fillLength, uint32_t dataLength)
{
  app->GetObject<TcpWebClient>()->SetFill (fill, fillLength, dataLength);
}

ApplicationContainer
TcpWebClientHelper::Install (Ptr<Node> node)
{
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
TcpWebClientHelper::Install (std::string nodeName)
{
  Ptr<Node> node = Names::Find<Node> (nodeName);
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
TcpWebClientHelper::Install (NodeContainer c)
{
  ApplicationContainer apps;
  for (NodeContainer::Iterator i = c.Begin (); i != c.End (); ++i)
    {
      Ptr<Node> node = *i;

      m_client = m_factory.Create<TcpWebClient> ();
      node->AddApplication (m_client);
      apps.Add (m_client);

    }

  return apps;
}

Ptr<Application>
TcpWebClientHelper::InstallPriv (Ptr<Node> node)
{
  m_client = m_factory.Create<TcpWebClient> ();
  node->AddApplication (m_client);

  return m_client;
}

Ptr<TcpWebClient>
TcpWebClientHelper::GetClient (void)
{
  return m_client;
}

} // namespace ns3
